/*
*
* Copyright (c) 2004 Sun Microsystems, Inc.
* All rights reserved.
* You may not modify, use, reproduce, or distribute this software except in compliance with the terms
* of the License at: http://developer.sun.com/berkeley_license.html
* Author : Sameer Tyagi, Sun Microsystems s.t@sun.com
* "Patterns and Strategies for Building Document-Based Web Services"
*  http://java.sun.com/developer/technicalArticles/xml/jaxrpcpatterns
*/
package com.examples.nodatabinding;

import com.examples.nodatabinding.po.LineItem;
import com.examples.nodatabinding.po.PurchaseOrder;
import com.examples.nodatabinding.postatus.ObjectFactory;
import com.examples.nodatabinding.postatus.Status;
import org.w3c.dom.Node;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.*;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

public class PurchaseOrderService implements IPurchaseOrder {

    public SOAPElement acceptPO(SOAPElement payload) throws POProcessingProblem, RemoteException {
        try {
            PurchaseOrder order = toObject(payload);
            System.out.println("JAXB Unmarshalled the purchase Order ");
            System.out.println("PurchaseOrder received in the web service");
            System.out.println("Order ID " + order.getPoID());
            System.out.println("Order creation date " + order.getCreateDate());
            System.out.println("Ship To Address " + order.getShipTo());
            System.out.println("Bill To Address " + order.getBillTo());
            System.out.println("Order Contents :");
            List items = order.getItems();
            Iterator it = items.iterator();
            while (it.hasNext()) {
                LineItem item = (LineItem) it.next();
                System.out.println("Item Name =" + item.getItemname() + " Price=" + item.getPrice() + " Quantity=" + item.getQuantity());
            }
            SOAPElement response = fromObject();
            return response;
        } catch (Exception e) {
            System.out.println("Exception in ceating a response" + e);
            throw new POProcessingProblem(createExceptionXML(e.getMessage()));
        }
    }

    /**
     * Unmarshall the element in the form of a SOAPElement to its JAXB object representation
     *
     * @param n
     * @throws Exception
     */
    public PurchaseOrder toObject(Node n) throws Exception {
        printNodeToConsole(n);
        JAXBContext jc = JAXBContext.newInstance("com.examples.nodatabinding.po");
        Unmarshaller u = jc.createUnmarshaller();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        Object o = u.unmarshal(n);
        // use the object that is generated by JAXB
        PurchaseOrder order = (PurchaseOrder) o;
        return order;
    }

    /**
     * Problem : How to marshall an object TO a SOAPElement.
     * Strategy1 : Marshall to a Document, then create a SOAPElement
     * This will work but is creating unncessary garbage. The SOAPBody object is never really used.
     * <code>
     * DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
     * Document doc = dbf.newDocumentBuilder().newDocument();
     * m.marshal( status, doc );
     * SOAPMessage message = MessageFactory.newInstance().createMessage();
     * SOAPBody body = message.getSOAPPart().getEnvelope().getBody();
     * SOAPElement element = body.addDocument(doc);
     * </pre>
     * Strategy 2 : Since SOAPElement should support org.w3c.dom interfaces it is possible to convert to
     * this using a dummy SOAPElement. Create a dummy element, marshall to it and then access the first child.
     * Thanks to Kohsuke Kawaguchi for the code below
     * <code>
     * SOAPElement parent = SOAPFactory.newInstance().createElement("dummy");
     * m.marshal( status, new DOMResult(parent) );
     * return (SOAPElement)parent.getChildElements().next();
     * </code>
     * This above segment, marshalls to teh following XML and then accesses the first child. <pre>
     * <dummy>
     * <ns0:Status xmlns="http://www.examples.com/types"
     * xsi:schemaLocation="http://www.examples.com/types PurchaseOrder.xsd">
     * <orderid xmlns="">ABC1080440067456</orderid>
     * <timestamp xmlns="">Sat Mar 27 21:14:27 EST 2004</timestamp>
     * </ns0:Status>
     * </dummy>
     * <p/>
     * </pre>
     *
     * @return
     * @throws Exception
     */
    public SOAPElement fromObject() throws Exception {
        // creating the ObjectFactory.
        ObjectFactory objFactory = new ObjectFactory();
        Status status = objFactory.createStatus();
        status.setTimestamp(new Date().toString());
        status.setOrderid("ABC" + System.currentTimeMillis());

        JAXBContext jc = JAXBContext.newInstance("com.examples.nodatabinding.postatus");
        Marshaller m = jc.createMarshaller();
        m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
        m.setProperty(Marshaller.JAXB_SCHEMA_LOCATION, "http://www.examples.com/types PurchaseOrder.xsd");
        m.marshal(status, System.out);
        /*
        At this point if unmarshalled the output will be
            <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
            <Status xsi:schemaLocation="http://www.examples.com/types PurchaseOrder.xsd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://www.examples.com/types">
                <orderid xmlns="">ABC1080439034862</orderid>
                <timestamp xmlns="">Sat Mar 27 20:57:14 EST 2004</timestamp>
            </Status>
       This can be printed to the console using :
               m.marshal( status, System.out );
       */
        SOAPElement parent = SOAPFactory.newInstance().createElement("dummy");
        m.marshal(status, new DOMResult(parent));
        return (SOAPElement) parent.getChildElements().next();

    }

    public static void printNodeToConsole(Node n) {
        try {
            System.out.println("=============================Request Received =========================================");
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();
            // also print to console
            transformer.transform(new DOMSource(n), new StreamResult(System.out));
            System.out.println("======================================================================");
        } catch (Exception e) {
            System.out.println(e);
        }
    }


    public static SOAPElement createExceptionXML(String msg) {
        try {
            SOAPFactory sf = SOAPFactory.newInstance();
            SOAPElement se = sf.createElement("message");
            se.addTextNode(msg);
            return se;
        } catch (Exception se) {
            // should never be called
            System.out.println("Exception creating Exception !!" + se);
            return null;
        }

    }

}
